<!----------------------------------------------------------------------------->
# Automatic layers integration
<!----------------------------------------------------------------------------->

**Contents:**
- [Automatic layers integration](#automatic-layers-integration)
  - [Source repositories used by BitBake](#source-repositories-used-by-bitbake)
  - [Integrating BSP changes](#integrating-bsp-changes)
  - [Merging project changes](#merging-project-changes)


## Source repositories used by BitBake

Some bitbake recipes, more precisely those regarding the Board Support Packages (BSPs) references, are set to 
`SRCREV = "${AUTOREV}"`. To enable reproducible builds, the revision for these recipes
is written to the `SRCREV.conf` file contained in the `conf` folder of the 
corresponding meta-layer. For instance, the [meta-seco-rk][4] 
meta-layer recipes contain the reference to the Rockhip Board Support Package (BSP) 
repositories and the corresponding [conf/SRCREV.conf][5] file looks something like this:

```ini
SRCREV:pn-u-boot-seco-rk = "7d205eec8c6d3e6d3e08390b3c276f9fafb67489"
SRCREV:pn-linux-seco-rk = "e6962a7c84438f36487ca8519f2b93b45efd59a6"
SRCREV:pn-linux-seco-px30-rk = "645936464a1d6641c2bab5686045d4fce281fdd3"
```

Each project is listed with its bitbake **recipe name** after `SRCREV:pn-` and
the fixed git **revision**.

[1]: https://git.seco.com/clea-os/seco-manifest
[2]: https://git.seco.com/clea-os/seco-manifest/-/blob/kirkstone/distro/seco-distro.xml
[3]: https://gerrit.googlesource.com/git-repo
[4]: https://git.seco.com/clea-os/layers/seco/meta-seco-rk
[5]: https://git.seco.com/clea-os/layers/seco/meta-seco-rk/-/blob/kirkstone/conf/SRCREV.conf

<!----------------------------------------------------------------------------->
## Integrating BSP changes
<!----------------------------------------------------------------------------->

Everytime a developer pushes a change to one of the projects referenced in the 
recipes, the corresponding `SRCREV.conf` file needs to get a new revision. 
In order for this to happen, a similar process to that described in the 
[Automatic manifest integration][6] document occurs. This process has been automated 
as part of our **CI pipeline** and runs automatically as soon as a 
**merge request is created** in one of the recipe repositories (i.e. currently 
those in the [`bsp` group][12]):

![Project merge request pipeline](recipe-mr-pipeline.png)

[6]: automatic-manifest-integration.md

The pipeline running in the recipe repositories is described in the `layers-integration.yml` [file][11].

[11]: https://git.seco.com/clea-os/infrastructure/gitlab-ci/-/blob/master/layers-integration.yml
[12]: https://git.seco.com/clea-os/bsp

The pipeline consists of three jobs:

1. **Integrate-into-layer**

   Create an integration branch on the meta-layer and update the project revision
   on it.

2. **Create-merge-request**

   Create a Merge Requeste (MR) in the meta-layer from the newly created integration branch into the main one. 
   The creation of this MR automatically triggers all the CI jobs described in detail in the 
   [Automatic manifest integration][6] document.

1. **Check**

   Check if the integration branch is up to date with the current meta-layer
   master branch (see [Automatic manifest integration][6] for why this is necessary).

See the following diagram for a visualization of this process:

![Create recipe merge request](create-recipe-mr.jpg)

The above workflow makes project changes **buildable** and **testable** in the
full manifest environment **before merging them**. Actually, the user could even 
[prevent the project merge requests from being merged unless the pipeline has run
successfully][7].

[7]: https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html

<!----------------------------------------------------------------------------->
## Merging project changes
<!----------------------------------------------------------------------------->

As soon as a pushed project change is ready to be merged, a developer can merge it. 
This triggers a **pipeline** with only a **single job**:

![Project master pipeline](project-master-pipeline.png)

1. **Merge**

   Merge the MR on the integration branch on the meta-layer created in the previous 
   step. This trigger a cascade process according to which a MR is opened and closed
   in the manifest (see [Automatic manifest integration][6]).