<!----------------------------------------------------------------------------->
# Automatic manifest integration
<!----------------------------------------------------------------------------->

**Contents:**
- [Automatic manifest integration](#automatic-manifest-integration)
  - [Manifest](#manifest)
  - [Integrating project changes](#integrating-project-changes)
      - [Skipping the build](#skipping-the-build)
  - [Merging project changes](#merging-project-changes)


## Manifest

The [Clea OS manifest][1] contains a list of projects which are used to build 
Edgehos OS distribution. A simplified version of a manifest file [`seco-distro.xml`][2]
(here containing only two projects) would look something like this:

```xml
<manifest>

    <default sync-j="8"/>

    <remote name="seco-repos"    alias="origin"         fetch="https://git.seco.com/clea-os" />
    <remote name="seco-layers"   alias="origin"         fetch="https://git.seco.com/clea-os/layers/seco" />

    <project name="meta-seco-clea-os-things" revision="901e92783ffb6c5a36fee99efd57cb0955b53a18" path="layers/meta-seco-clea-os-things"       remote="seco-layers" />
    <project name="meta-seco-clea-os-embedded" revision="8fe5d96c85605645260baa6e78b198a3b69f607e" path="layers/meta-seco-clea-os-embedded" remote="seco-layers" />

</manifest>

```

Each project is listed with a **name**, a local **path** where the [repo][3]
tool will clone the source code to, and a fixed git **revision**. We do not use
a branch name here in order to have a defined, taggable manifest state at all
times.

[1]: https://git.seco.com/clea-os/seco-manifest
[2]: https://git.seco.com/clea-os/seco-manifest/-/blob/kirkstone/distro/seco-distro.xml
[3]: https://gerrit.googlesource.com/git-repo
[4]: https://git.seco.com/clea-os/layers/seco/meta-seco-rk
[5]: https://git.seco.com/clea-os/layers/seco/meta-seco-rk/-/blob/kirkstone/conf/SRCREV.conf

<!----------------------------------------------------------------------------->
## Integrating project changes
<!----------------------------------------------------------------------------->

Everytime a developer pushes a change to one of the projects contained in the
manifest, the project gets a new git revision. In
order to have this change integrated into the manifest, the according `revision`
attribute has to be updated. This process has been automated as part of our **CI
pipeline** and runs automatically as soon as a **merge request is created**:

![Project merge request pipeline](project-mr-pipeline.png)

The pipeline running in the meta-layers is described in the `manifest-integration.yml` [file][11].

[11]:https://git.seco.com/clea-os/infrastructure/gitlab-ci/-/blob/master/manifest-integration.yml

The pipeline mainly consists of three jobs:

1. **Integrate**

   Create an integration branch on the manifest and update the project revision
   on it.

2. **Build**

   Trigger a build on the integration branch, which contains the change.
   This uses GitLab's [multi-project pipeline][6] functionality, which adds a
   separate *Downstream* pipeline running on the manifest repository (seen on
   the right).

3. **Check**

   Check if the integration branch is up to date with the current manifest
   master (see ["Retrigger" job below][8] for why this is necessary).

See the following diagram for a visualization of this process:

![Create layer merge request](create-layer-mr.jpg)

The above workflow makes project changes **buildable** and **testable** in the
full manifest environment **before merging them**. Actually, the user could even 
[prevent the project merge requests from being merged unless the pipeline has run
successfully][7].

[6]: https://docs.gitlab.com/ee/ci/pipelines/multi_project_pipelines.html
[7]: https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html
[8]: #merging-project-changes

**NOTE**: In the current implementation, the build/deploy/test stages are left to
be triggered manually whenever a MR is created in order to not overload the build
server.
#### Skipping the build

In the case of changes that are not build-relevant or have a rather low
probability of breaking the build (for example documentation), the developer has
the option to skip the build by adding the `skip build` label to a merge request
on its creation. The `build` and `check` stages of the pipeline are left out
then, and the merge request can be merged immediately after the `integrate`
stage has completed.

<!----------------------------------------------------------------------------->
## Merging project changes
<!----------------------------------------------------------------------------->

As soon as a pushed project change is ready to be merged (i.e. the pipeline has
successfully run on the merge request), a developer can merge it. This triggers
a **pipeline** with only a **single job**:

![Project master pipeline](project-master-pipeline.png)

1. **Merge**

   Create a merge request on the manifest, which merges the project integration
   branch into the master branch, and merge it.

This will however automatically trigger **another pipeline** on the **manifest
master branch**, since there is a new commit now:

![Manifest master pipeline](manifest-master-pipeline.png)

This pipeline consists of two jobs:

1. **Retrigger**

   Retrigger the *"Check"* job on all open merge requests of all projects
   contained in the manifest. This will invalidate the pipeline on these merge
   requests. The manifest has a new commit now, so the already completed
   pipelines on the open merge requests have run on a now outdated manifest
   state.

2. **Build**

   Check if a build has already run for this commit. This is usually true,
   because every merge request triggers a complete build. If so, just mirror
   the build result from there.
   There are corner cases however in which no pipeline has run in the merge
   request, e.g. when two project merge requests from different projects are
   merged simultaneously. In this case, perform a complete build on the commit.

See the following diagram for a visualization of this process:

![Merge layer merge request](merge-layer-mr.jpg)
