<!----------------------------------------------------------------------------->
# Automatic tools integration
<!----------------------------------------------------------------------------->

**Contents:**
- [Automatic tools integration](#automatic-tools-integration)
  - [Tools repositories used by `seco-base`](#tools-repositories-used-by-seco-base)
  - [Integrating tools repository changes](#integrating-tools-repository-changes)
  - [Merging project changes](#merging-project-changes)


## Tools repositories used by `seco-base`

The [`seco-base` repository][1] in the Clea OS project contains the scripts and the tools to properly configure Yocto environment. In order for these tools to work, the repository also leverages the source code contained in some of the repositories in the [`tools` group][2]. For instance, in the [`kirkstone` branch][3], `seco-base` imports as submodule the source code contained in the [`menuconfig`][4] repository. For the `scarthgap` branch, a different approach is followed and the `seco-base` repository imports as submodule the [`seco-build-configurator` repository][5].  
For this reason, it was necessary to introduce an automation that allows, whenever a change is introduced in one of the repositories imported as a submodule, that this change is integrated into the `seco-base` repository or more generally into other repositories within the Clea OS `tools` group. 

![seco-base submodules](seco-base-submodules.png)

In order to realise this automatism, a structure quite similar to the one already introduced for the automatic integration of changes on SRCREV.conf files within layers was implemented (see [Automatic layers integration][6] documentation).

[1]: https://git.seco.com/clea-os/seco-base
[2]: https://git.seco.com/clea-os/tools
[3]: https://git.seco.com/clea-os/seco-base/-/tree/kirkstone?ref_type=heads
[4]: https://git.seco.com/clea-os/tools/menuconfig
[5]: https://git.seco.com/clea-os/tools/seco-build-configurator

<!----------------------------------------------------------------------------->
## Integrating tools repository changes
<!----------------------------------------------------------------------------->

Everytime a developer pushes a change to one of the projects referenced in the 
submodule, the corresponding commit SHA needs to get a new revision. 
In order for this to happen, a similar process to that described in the 
[Automatic layers integration][6] document occurs. This process has been automated 
as part of our **CI pipeline** and runs automatically as soon as a 
**merge request is created** in one of the imported-as-submodule repositories (i.e. currently 
those in the [`tools` group][2]):

![Submodule merge request pipeline](seco-base-mr-pipeline.png)

The pipeline running in the submodule repositories is described in the `tools-integration.yml` [file][7], 
is based on the [Dynamic Child Pipelines][8] mechanism and consists of three main jobs:

1. **generate-pipelines**

   Creates a `.yml` file for a downstream pipeline that actually performs all significant automation
   tasks. The pipeline described in that file is then triggered by the `trigger-pipelines` job in the 
   context of a [Dynamic Child Pipelines][8] mechanism.

2. **integrate-into-tools**

   Create an integration branch on the `seco-base` repository and update the corresponding submodule SHA.

3. **create-merge-request**

   Create a Merge Requeste (MR) in the `seco-base` repostiory from the newly created integration 
   branch into the main one. The creation of this MR automatically triggers all the CI jobs 
   described in detail in the [Automatic manifest integration][9] document.

See the following diagram for a visualization of this process:

![Create tools merge request](create-tools-mr.jpg)

The above workflow makes project changes **buildable** and **testable** in the
full manifest environment **before merging them**. Actually, the user could even 
[prevent the project merge requests from being merged unless the pipeline has run
successfully][10].

[6]: automatic-layers-integration.md
[7]: https://git.seco.com/clea-os/infrastructure/gitlab-ci/-/blob/master/tools-integration.yml
[8]: https://docs.gitlab.com/ee/ci/pipelines/parent_child_pipelines.html#dynamic-child-pipeline-example
[9]: automatic-manifest-integration.md
[10]: https://docs.gitlab.com/ee/user/project/merge_requests/merge_when_pipeline_succeeds.html

<!----------------------------------------------------------------------------->
## Merging project changes
<!----------------------------------------------------------------------------->

As soon as a pushed project change is ready to be merged, a developer can merge it. 
This triggers a **pipeline** based on the [Dynamic Child Pipelines][8] mechanism 
with one significant job:

![Project master pipeline](tools-mr-merge.png)

1. **merge-into-tools**

   Merge the MR created in the previous step in `seco-base` project. 
   This trigger a cascade process according to which a MR is opened and merged
   in the manifest (see [Automatic manifest integration][9]).