<!----------------------------------------------------------------------------->
# `gitlab-ci` deployment
<!----------------------------------------------------------------------------->

<!----------------------------------------------------------------------------->
## Deploy changes into all projects using 'gitlab-ci' as submodule
<!----------------------------------------------------------------------------->

The gitlab-ci repo is integrated into the other related projects as a submodule
and as reference in their `.gitlab-ci.yml` file. Both references need to be
updated when a change in the repo shall be deployed to the projects using it.

To achieve this the CI pipeline is set up to update all these projects in one
run. The submodule *does not* need to be updated manually, but the CI needs some
manual steps to do the deployment.
As this repo contains all the CI code, some extra care is needed in order not to
break the whole thing. To allow testing the CI before everything is merged into
the Yocto pipeline, a sandbox environment in the form of a set of test projects is
available in the [ci-test group][1].

[1]: https://git.seco.com/clea-os/infrastructure/ci-test


<!----------------------------------------------------------------------------->
## Workflow to deploy a change
<!----------------------------------------------------------------------------->

* Commit and push change to the gitlab-ci repo, create a merge request.

* The pipeline checks syntax and other stuff in the *Analyze* stage.

* The *Integrate* stage allows testing the change in the complete setup.
  There are two possible integrate steps, one to deploy the change into Merge
  Request in the *ci-test* group, the other is for the *productive yocto* setup.

* When the integrate stage has run, a build is started in the related manifest
  project.

* In all projects there are now integration branches created, named
  `integrate/gitlab-ci/<gitlab-ci branch name>/into/<project branch name>`.
  These could be used to add additional changes if needed. But that should only
  happen in very special cases.

* If everything is OK until here, (build OK, CI works as expected ...)
  the change can be merged into the main branch of gitlab-ci.
  *This only adds the patch to the gitlab-ci repo's main branch, the reference
    in the projects using this are not yet updated*.

* After the merge a new pipeline is created. In this pipeline there are 
  manual jobs created that allows to merge the gitlab-ci integration branches into 
  the project main branches. These jobs update the reference in the project 
  groups *'clea-os/infrastructure/ci-test'* and *'clea-os'*, as well as on all 
  custom projects inside *'clea-os/custom'* group. This is split up to 
  allow more testing in cases where the CI changes can only be tested 
  when working on the main branches.

* When the behaviour of the CI on the main branches may be effected of the
  change only *'merge-ci-test'* should be executed. 
  After this some tests may be done on the ci-test projects.
  Probably you want to create a commit in *minimal-foo*, create a merge-request,
  check the running pipeline, and merge it.
  After that, check if the merge is succesfull and the manifest is updated as 
  needed.

* When everything is working, all the other pipelines allowing to merge the results 
  into the various Clea OS projects (both generic and customs) can be executed
  to deploy the change to the productive projects.

* When the tests on *'ci-test'* have failed, just create a new patch fixing the
  issue in the gitlab-ci repo and do the testing again.

## Setup groups for integrations

The pipeline to deploy Gitlab CI/CD workflow to projects is intended to work in 
the same way both for generic and custom projects in the Clea OS group. In order to 
allow modifications to be deployed to all these projects in a scalable way, the 
`INTEGRATION_CI` variable is defined at project level:

* Go to `Settings->CI/CD` for this project.

* Click on *Variables* and check for `INTEGRATION_CI` variable. This contains a list 
  of lines each one specifying which group, manifest project and main branch to deploy 
  the modifications to.

  Example: `clea-os:seco-manifest:kirkstone`
  
  * `clea-os`: The group on which the changes shall be deployed.
  * `seco-manifest`: The name of the manifest project within that group.
  * `kirkstone`: The main branch of the Yocto manifest to be used for the
                 integration.

  There may be multiple lines in the `INTEGRATION_CI` variable to integrate into
  several groups, manifests or branches.


