#!/bin/bash

# SDK build
if [[ "$BITBAKE_TASK" == "populate_sdk" ]]; then
  count_manifest=`ls -1 $SDK_PATH/*.manifest 2>/dev/null | wc -l`
  count_sh=`ls -1 $SDK_PATH/*.sh 2>/dev/null | wc -l`
  count_json=`ls -1 $SDK_PATH/*.json 2>/dev/null | wc -l`

  if [[ $count_manifest -lt 2 || $count_sh -lt 1 || $count_json -lt 1 ]]; then
    echo "Not all SDK artifacts were found, exiting with error 1 .."
    exit 1
  else
    echo "All SDK artifacts were found!"
  fi
else
# Image build
  ARTIFACTS_DIR=$(dirname $(ls -1 $BASEDIR/$ARTIFACTS_PATH | head -1))

  count_uboot_files=0
  count_image=`ls -1 $ARTIFACTS_DIR/$KERNEL_FILE 2>/dev/null | wc -l`
  count_dtb=`ls -1 $ARTIFACTS_DIR/*.dtb 2>/dev/null | wc -l`
  count_dtbo=`ls -1 $ARTIFACTS_DIR/*.dtbo 2>/dev/null | wc -l`
  count_fs=`ls -1 $ARTIFACTS_DIR/*.$FS_EXTENSION 2>/dev/null | wc -l`
  count_wic=`ls -1 $ARTIFACTS_DIR/*.$IMAGE_EXTENSION 2>/dev/null | wc -l`
  count_bundle=`ls -1 $ARTIFACTS_DIR/*.raucb 2>/dev/null | wc -l`

  # Check custom artifacts
  if [ -n "$CUSTOM_ARTIFACTS" ]; then
    echo "CUSTOM_ARTIFACTS: $CUSTOM_ARTIFACTS"
    count_el=0
    for custom_artifact in $CUSTOM_ARTIFACTS; do
      c_artifacts=`ls -1 $ARTIFACTS_DIR/$custom_artifact 2>/dev/null | wc -l`
      count_el=$((count_el+$c_artifacts))
    done
    if [ $count_el -gt 0 ]; then
        echo "$count_el custom artifacts were found!"
    else
        echo "No custom artifacts were found, warning!"
    fi
  fi

  if [[ $UBOOT_FILE != *"none"* ]]; then
    count_el=0
    for i in $UBOOT_FILE; do
      count_tmp=0
      count_tmp=`ls -1 $ARTIFACTS_DIR/$i 2>/dev/null | wc -l`
      count_uboot_files=$(($count_uboot_files+$count_tmp))
      count_el=$((count_el+1))
    done
    if [ $count_uboot_files -lt "$count_el" ]; then
      echo "U-Boot artifacts lower than expected found, warning!"
    else
      echo "All uboot artifacts were found!"
    fi
  fi

  if [[ $KERNEL_FILE != *"none"* ]]; then
    if [ $count_image -eq 0 ]; then
      echo "No kernel image artifacts were compiled, exiting with error 1 .."
      exit 1
    else
      echo "Kernel image artifacts were found!"
    fi

    if [ $count_dtb -eq 0 ]; then
      echo "No dtb artifacts were compiled, exiting with error 1 .."
      exit 1
    else
      echo "dtb artifacts were found!"
    fi

    if [ $count_dtbo -eq 0 ]; then
      echo "No dtbo artifacts were compiled, warning!"
    else
      echo "dtbo artifacts were found!"
    fi
  fi

  if [ $count_fs -eq 0 ]; then
      echo "No filesystem artifacts were compiled, exiting with error 1 .."
      exit 1
  else
      echo "Filesystem artifacts were found!"
  fi

  if [ $count_wic -eq 0 ]; then
    echo "No image artifacts were compiled, exiting with error 1 .."
    exit 1
  else
    echo "Image artifacts were found!"
  fi

  if [[ "$CI_JOB_NAME" != *"embedded"* ]]; then
    if [ $count_bundle -eq 0 ]; then
      echo "No bundle artifacts were compiled, exiting with error 1 .."
      exit 1
    else
      echo "Bundle artifacts were found!"
    fi
  fi
fi
