#!/bin/ash

BASE_DIRECTORY=$1
ART_PATH=$2
DEPLOY_PATH=$3
AZURE_STORAGE_SAS_TOKEN=$4
AZURE_CONTAINER_NAME=$5
AZURE_PATH=$6
MD5_SUMS_FILE=$7

generate_md5() {
    local file_path="$1"
    if [[ -f "$file_path" ]]; then
        summ=$(md5sum $file_path | cut -d ' ' -f 1)
        echo "$summ   $(basename $file_path)">> $MD5_SUMS_FILE
        echo "MD5 hash for ${file_path} created."
    else
        echo "Could not create md5 hash. File not found: $file_path !"
    fi
}

# SDK build
if [[ "$BITBAKE_TASK" == "populate_sdk" ]]; then

    echo "##################################################################"
    echo "#######################  SDK FILES  ############################"
    echo "##################################################################"

    #Copy the bundle file and rename it
    tar -czvf $BASE_DIRECTORY/$ART_PATH/$sdk_filename -C $BASE_DIRECTORY/$ART_PATH .
    generate_md5 $BASE_DIRECTORY/$ART_PATH/$sdk_filename

    az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
        --sas-token $AZURE_STORAGE_SAS_TOKEN \
        --container-name $AZURE_CONTAINER_NAME \
        --file $BASE_DIRECTORY/$ART_PATH/$sdk_filename \
        --name $AZURE_PATH/$DEPLOY_PATH/$sdk_filename \
        --overwrite
    echo "Finishing SDK upload"

else
# Image build
    if [[ ! "${CI_JOB_NAME}" =~ "embedded" ]]; then

        echo "##################################################################"
        echo "#######################  BUNDLE FILE  ############################"
        echo "##################################################################"

        #Copy the bundle file and rename it
        find $ART_PATH -iname *"$MACHINE.raucb" -exec cp -L {} $BASE_DIRECTORY/ \;
        mv *raucb $BASE_DIRECTORY/$bundle_filename
        generate_md5 $BASE_DIRECTORY/$bundle_filename
        az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
            --sas-token $AZURE_STORAGE_SAS_TOKEN \
            --container-name $AZURE_CONTAINER_NAME \
            --file $BASE_DIRECTORY/$bundle_filename \
            --name $AZURE_PATH/$DEPLOY_PATH/$bundle_filename \
            --overwrite

    fi

    case none in
    *${UBOOT_FILE}*) echo "Not deploying U-Boot for this board since not present" ;;
    *)
        echo "##################################################################"
        echo "########################  UBOOT  #################################"
        echo "##################################################################"

        #Compress the uboot package and create a tar.gz file
        echo $ART_PATH
        ls -la $ART_PATH
        uboot_dir="$BASE_DIRECTORY/seco-uboot"
        mkdir -p $uboot_dir

        for el in $UBOOT_FILE; do
        find $ART_PATH -iname $el -exec cp -L {} $uboot_dir/ \;
        done
        if [ -z "$(ls -A $uboot_dir 2>/dev/null)" ]; then
            echo "No U-Boot artifacts found, not creating archive to deploy!"
        else
            tar -czvf $BASE_DIRECTORY/$uboot_filename -C $uboot_dir $(for el in ${UBOOT_FILE}; do echo $el; done)
            generate_md5 $BASE_DIRECTORY/$uboot_filename
            ls -la

            az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
                --sas-token $AZURE_STORAGE_SAS_TOKEN \
                --container-name $AZURE_CONTAINER_NAME \
                --file $BASE_DIRECTORY/$uboot_filename \
                --name $AZURE_PATH/$DEPLOY_PATH/$uboot_filename \
                --overwrite

            echo "Finishing U-Boot upload"
        fi
        ;;
    esac

    case none in
    *${KERNEL_FILE}*) echo "Not deploying Kernel for this board since not present" ;;
    *)
        echo "##################################################################"
        echo "########################  KERNEL  ################################"
        echo "##################################################################"

        kernel_dir="$BASE_DIRECTORY/seco-kernel-$BOARD-$PROCESSOR"
        mkdir -p $kernel_dir/boot $kernel_dir/lib

        #Find all the needed kernel packages and copy them inside the proper folders
        find $BASE_DIRECTORY/$ART_PATH -iname $KERNEL_FILE \
            -exec cp -L {} $kernel_dir/boot \;
        find $BASE_DIRECTORY/$ART_PATH -maxdepth 2 -iname *.dtb \
            -exec cp -L {} $kernel_dir/boot \;
        find $BASE_DIRECTORY/$ART_PATH -iname *.dtbo \
            -exec cp -L {} $kernel_dir/boot \;
        find $BASE_DIRECTORY/$ART_PATH -iname *fw*.bin \
            -exec cp -L {} $kernel_dir/boot \;
        find $BASE_DIRECTORY/$ART_PATH -iname modules-seco-*.tgz \
            -exec cp -L {} $BASE_DIRECTORY/temp_mod.tgz \;
        tar -xf $BASE_DIRECTORY/temp_mod.tgz -C $kernel_dir
        rm $BASE_DIRECTORY/temp_mod.tgz

        #Compress the kernel package and create a tar.gz file
        tar -czvf $BASE_DIRECTORY/$kernel_filename -C $kernel_dir boot lib
        generate_md5 $BASE_DIRECTORY/$kernel_filename
        rm -rf $kernel_dir
        az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
            --sas-token $AZURE_STORAGE_SAS_TOKEN \
            --container-name $AZURE_CONTAINER_NAME \
            --file $BASE_DIRECTORY/$kernel_filename \
            --name $AZURE_PATH/$DEPLOY_PATH/$kernel_filename \
            --overwrite

        echo "Finishing Kernel upload"
        ;;
    esac

    echo "##################################################################"
    echo "######################  FILESYSTEM  ##############################"
    echo "##################################################################"

    # Copy the filesystem file and rename it
    find $BASE_DIRECTORY/$ART_PATH -iname *."$FS_EXTENSION" \
        -exec cp -L {} $BASE_DIRECTORY/filesystemTMP."$FS_EXTENSION" \;
    mv filesystemTMP."$FS_EXTENSION" $BASE_DIRECTORY/$filesystem_name
    generate_md5 $BASE_DIRECTORY/$filesystem_name
    az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
        --sas-token $AZURE_STORAGE_SAS_TOKEN \
        --container-name $AZURE_CONTAINER_NAME \
        --file $BASE_DIRECTORY/$filesystem_name \
        --name $AZURE_PATH/$DEPLOY_PATH/$filesystem_name \
        --overwrite

    echo "################################################################"
    echo "#######################  BMAP IMAGE  ###########################"
    echo "################################################################"

    #Copy the BMAP .wic image and rename it
    find $BASE_DIRECTORY/$ART_PATH \
        -iname *rootfs."$IMAGE_EXTENSION" \
        -exec cp -L {} . \;
    mv *rootfs."$IMAGE_EXTENSION" $BASE_DIRECTORY/$image_name
    generate_md5 $BASE_DIRECTORY/$image_name
    az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
        --sas-token $AZURE_STORAGE_SAS_TOKEN \
        --container-name $AZURE_CONTAINER_NAME \
        --file $BASE_DIRECTORY/$image_name \
        --name $AZURE_PATH/$DEPLOY_PATH/$image_name \
        --overwrite

    echo "################################################################"
    echo "#######################  BMAP FILE  ############################"
    echo "################################################################"

    #Copy the bmap files and rename it
    find $ART_PATH -iname *rootfs.wic.bmap -exec cp -L {} $BASE_DIRECTORY/ \;
    mv *rootfs.wic.bmap $BASE_DIRECTORY/$bmap_name
    generate_md5 $BASE_DIRECTORY/$bmap_name
    az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
        --sas-token $AZURE_STORAGE_SAS_TOKEN \
        --container-name $AZURE_CONTAINER_NAME \
        --file $BASE_DIRECTORY/$bmap_name \
        --name $AZURE_PATH/$DEPLOY_PATH/$bmap_name \
        --overwrite

    echo "################################################################"
    echo "###################  Custom artifacts  #########################"
    echo "################################################################"

    if [ -n "$CUSTOM_ARTIFACTS" ]; then
        echo "CUSTOM_ARTIFACTS: $CUSTOM_ARTIFACTS"
        custom_artifacts_temp_dir="custom_artifacts_temp_dir"
        mkdir -p $BASE_DIRECTORY/$custom_artifacts_temp_dir

        for custom_artifact in $CUSTOM_ARTIFACTS; do
            cp -rL $BASE_DIRECTORY/$ART_PATH/$custom_artifact $BASE_DIRECTORY/$custom_artifacts_temp_dir/
        done
        tar -czvf $BASE_DIRECTORY/$custom_artifacts_temp_dir/$custom_artifacts_filename -C $BASE_DIRECTORY/$custom_artifacts_temp_dir .
        generate_md5 $BASE_DIRECTORY/$custom_artifacts_temp_dir/$custom_artifacts_filename

        az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
            --sas-token $AZURE_STORAGE_SAS_TOKEN \
            --container-name $AZURE_CONTAINER_NAME \
            --file $BASE_DIRECTORY/$custom_artifacts_temp_dir/$custom_artifacts_filename \
            --name $AZURE_PATH/$DEPLOY_PATH/$custom_artifacts_filename \
            --overwrite

        rm -rf $custom_artifacts_temp_dir
    fi

fi

echo "################################################################"
echo "########################  MD5 sums  ############################"
echo "################################################################"

# Check if the file exists
md5summs_exists=$(az storage blob exists \
    --account-name $AZURE_STORAGE_ACCOUNT \
    --sas-token $AZURE_STORAGE_SAS_TOKEN \
    --container-name $AZURE_CONTAINER_NAME \
    --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename \
    --query exists)

if [ "$md5summs_exists" = "true" ]; then
    echo "MD5 checksums already deployed. Updating data in the existing file."
    echo "Downloading existing file..."
    az storage blob download \
      --account-name $AZURE_STORAGE_ACCOUNT \
      --sas-token $AZURE_STORAGE_SAS_TOKEN \
      --container-name $AZURE_CONTAINER_NAME \
      --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename \
      --file $MD5_SUMS_FILE.old \

    echo "Updating new data..."
    # Update checksumm in file if filename is already in the list or add new checksum if not
    cat $MD5_SUMS_FILE $MD5_SUMS_FILE.old | awk '!seen[$2]++' > $MD5_SUMS_FILE.temp
    mv $MD5_SUMS_FILE.temp $MD5_SUMS_FILE
fi

echo "Upload MD5 checksumms."
az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
    --sas-token $AZURE_STORAGE_SAS_TOKEN \
    --container-name $AZURE_CONTAINER_NAME \
    --file $MD5_SUMS_FILE \
    --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename \
    --overwrite
