#!/bin/ash

DEPLOY_PATH=$1
AZURE_STORAGE_SAS_TOKEN=$2
AZURE_CONTAINER_NAME=$3
AZURE_PATH=$4

if [ "$CI_JOB_STATUS" == "success" ]; then

    # Get the current year and month
    current_year=$(date -u '+%Y')
    current_month=$(date -u '+%m')

    # Remove month leading zero if present to avoid
    # octal interpretation of numbers
    current_month=${current_month#0}

    # Calculate the token expiration month and year
    future_month=$(( (current_month + 6) % 12 ))
    future_year=$(( current_year + (current_month + 6) / 12 ))

    # Adjust the year if the future month is 0
    if [ $future_month -eq 0 ]; then
        future_month=12
        future_year=$(( future_year - 1 ))
    fi

    # Format the expiration date
    expire_date="${future_year}-${future_month}-01T00:00Z"

    if [[ "$BITBAKE_TASK" == "populate_sdk" ]]; then

      # Generate read-only SAS token with 6-months expiration policy
      sas_sdk=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$sdk_filename --permissions r --expiry "$expire_date" --output tsv)
      url_sdk=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$sdk_filename --output tsv | sed -E 's/\?s.*//')
      sas_md5sums=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename --permissions r --expiry "$expire_date" --output tsv)
      url_md5sums=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename --output tsv | sed -E 's/\?s.*//')

      artifacts="sdk md5sums"

    else

      if [[ ! "${CI_JOB_NAME}" =~ "embedded" ]]; then
          # Generate read-only SAS token with 6-months expiration policy
          sas_bundle=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$bundle_filename --permissions r --expiry "$expire_date" --output tsv)
          url_bundle=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$bundle_filename --output tsv | sed -E 's/\?s.*//')
      fi

      case none in
      *${UBOOT_FILE}*) echo "Not deployed U-Boot for this board since not present" ;;
      *)
        # Generate read-only SAS token with 6-months expiration policy
        sas_uboot=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$uboot_filename --permissions r --expiry "$expire_date" --output tsv)
        url_uboot=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$uboot_filename --output tsv | sed -E 's/\?s.*//')
        ;;
      esac

      case none in
      *${KERNEL_FILE}*) echo "Not deployed Kernel for this board since not present" ;;
      *)
        # Generate read-only SAS token with 6-months expiration policy
        sas_kernel=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$kernel_filename --permissions r --expiry "$expire_date" --output tsv)
        url_kernel=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$kernel_filename --output tsv | sed -E 's/\?s.*//')
        ;;
      esac

      # Generate read-only SAS token with 6-months expiration policy
      sas_filesystem=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$filesystem_name --permissions r --expiry "$expire_date" --output tsv)
      url_filesystem=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$filesystem_name --output tsv | sed -E 's/\?s.*//')
      sas_image=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$image_name --permissions r --expiry "$expire_date" --output tsv)
      url_image=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$image_name --output tsv | sed -E 's/\?s.*//')
      sas_bmap=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$bmap_name --permissions r --expiry "$expire_date" --output tsv)
      url_bmap=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$bmap_name --output tsv | sed -E 's/\?s.*//')
      sas_md5sums=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename --permissions r --expiry "$expire_date" --output tsv)
      url_md5sums=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$md5sums_filename --output tsv | sed -E 's/\?s.*//')

      artifacts="bundle uboot kernel filesystem image bmap md5sums"

      if [ -n "$CUSTOM_ARTIFACTS" ]; then
          # Generate read-only SAS token with 6-months expiration policy
          sas_custom=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$custom_artifacts_filename --permissions r --expiry "$expire_date" --output tsv)
          url_custom=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name $AZURE_PATH/$DEPLOY_PATH/$custom_artifacts_filename --output tsv | sed -E 's/\?s.*//')
          artifacts="$artifacts custom"
      fi

    fi

    case "$AZURE_CONTAINER_NAME" in
        *"private"*)
          for i in $artifacts; do
            if [ -n "\$url_$i" ]; then
                eval echo -e "\$url_$i?\$sas_$i" >> Link_report_"$BOARD"_"$IMAGE_NAME".txt
            fi
          done
          ;;
        *)
          for i in $artifacts; do
            if [ -n "\$url_$i" ]; then
                eval echo -e "\$url_$i" >> Link_report_"$BOARD"_"$IMAGE_NAME".txt
            fi
          done
          ;;
    esac

else
    echo -e "JOBS FAILED" >>Link_report_"$BOARD"_"$IMAGE_NAME".txt
fi
