#!/bin/ash

AZURE_PATH=$1
TAG_NAME=$2                 
AZURE_STORAGE_SAS_TOKEN=$3
AZURE_CONTAINER_NAME=$4

cat Job_message.txt Job_message_*.txt > Job_message_${SECO_REMOTE}_${TAG_NAME}.txt
cat Job_report_*.txt > Job_report_${SECO_REMOTE}_${TAG_NAME}.txt
cat Link_report_*.txt > Link_report_${SECO_REMOTE}_${TAG_NAME}.txt

cat changelog.md > Changelog_${SECO_REMOTE}_${TAG_NAME}.md

sed -i -E 's/\"//g' Link_report_${SECO_REMOTE}_${TAG_NAME}.txt

az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
    --sas-token $AZURE_STORAGE_SAS_TOKEN \
    --container-name $AZURE_CONTAINER_NAME \
    --file Job_report_${SECO_REMOTE}_${TAG_NAME}.txt \
    --name ${AZURE_PATH}/Job_report_${SECO_REMOTE}_${TAG_NAME}.txt \
    --overwrite

az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
    --sas-token $AZURE_STORAGE_SAS_TOKEN \
    --container-name $AZURE_CONTAINER_NAME \
    --file Link_report_${SECO_REMOTE}_${TAG_NAME}.txt \
    --name ${AZURE_PATH}/Link_report_${SECO_REMOTE}_${TAG_NAME}.txt \
    --overwrite

az storage blob upload --account-name $AZURE_STORAGE_ACCOUNT \
    --sas-token $AZURE_STORAGE_SAS_TOKEN \
    --container-name $AZURE_CONTAINER_NAME \
    --file Changelog_${SECO_REMOTE}_${TAG_NAME}.md \
    --name ${AZURE_PATH}/Changelog_${SECO_REMOTE}_${TAG_NAME}.md \
    --overwrite

cp .gitlab-ci/notification.json .

case "$AZURE_CONTAINER_NAME" in 
    *"private"*)
      # Get the current year and month
      current_year=$(date -u '+%Y')
      current_month=$(date -u '+%m')

      # Remove month leading zero if present to avoid
      # octal interpretation of numbers
      current_month=${current_month#0} 

      # Calculate the token expiration month and year
      future_month=$(( (current_month + 6) % 12 ))
      future_year=$(( current_year + (current_month + 6) / 12 ))
      
      # Adjust the year if the future month is 0
      if [ $future_month -eq 0 ]; then
          future_month=12
          future_year=$(( future_year - 1 ))
      fi
          
      # Format the expiration date
      expire_date="${future_year}-${future_month}-01T00:00Z"

      # Get read-only blob SAS tokens
      sas_job_report=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Job_report_${SECO_REMOTE}_${TAG_NAME}.txt --permissions r --expiry "$expire_date" --output tsv)
      sas_link_report=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Link_report_${SECO_REMOTE}_${TAG_NAME}.txt --permissions r --expiry "$expire_date" --output tsv)
      sas_changelog_report=$(az storage blob generate-sas --account-key $AZURE_STORAGE_KEY --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Changelog_${SECO_REMOTE}_${TAG_NAME}.md --permissions r --expiry "$expire_date" --output tsv)

      # Get URLs without SAS container token, which shall be replaced with the blob one
      job_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Job_report_${SECO_REMOTE}_${TAG_NAME}.txt --output tsv | sed -E 's/\?s.*//')
      link_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Link_report_${SECO_REMOTE}_${TAG_NAME}.txt --output tsv | sed -E 's/\?s.*//')
      changelog_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Changelog_${SECO_REMOTE}_${TAG_NAME}.md --output tsv | sed -E 's/\?s.*//')

      #Construct final URLs
      job_report="$job_report?$sas_job_report"
      link_report="$link_report?$sas_link_report"
      changelog_report="$changelog_report?$sas_changelog_report"
      ;;
    *)
      # Get URLs without SAS container token
      job_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Job_report_${SECO_REMOTE}_${TAG_NAME}.txt --output tsv | sed -E 's/\?s.*//')
      link_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Link_report_${SECO_REMOTE}_${TAG_NAME}.txt --output tsv | sed -E 's/\?s.*//')
      changelog_report=$(az storage blob url --container-name $AZURE_CONTAINER_NAME --name ${AZURE_PATH}/Changelog_${SECO_REMOTE}_${TAG_NAME}.md --output tsv | sed -E 's/\?s.*//')
      ;;
esac

job_report="$(echo $job_report | sed -E 's/\s.*$//')"
jq --arg job_var $job_report '.potentialAction[0].targets[0].uri = $job_var' notification.json >notification_1.json

link_report="$(echo $link_report | sed -E 's/\s.*$//')"
jq --arg link_var $link_report '.potentialAction[1].targets[0].uri = $link_var' notification_1.json >notification_2.json

changelog_report="$(echo $changelog_report | sed -E 's/\s.*$//')"
jq --arg changelog_var $changelog_report '.potentialAction[2].targets[0].uri = $changelog_var' notification_2.json >notification_3.json
sed -i -E 's/\\"//g' notification_3.json

jq --arg text "$(cat Job_message_${SECO_REMOTE}_${TAG_NAME}.txt)" '.text = $text' notification_3.json >notification_4.json
sed -i -E 's/\\n/ \\n\\n /g' notification_4.json

curl -X POST -H "Content-Type: application/json" --data @notification_4.json $WEBHOOK

cp Link_report_${SECO_REMOTE}_${TAG_NAME}.txt Link_report.txt
sed -i 's/\"//g' Link_report.txt

echo "$changelog_report" >changelog_link.txt
